import gevent
import yum

from aj.api import *
from aj.plugins.packages.api import PackageManager, Package


@component(PackageManager)
class YUMPackageManager (PackageManager):
    id = 'yum'
    name = 'YUM'

    def __init__(self, context):
        PackageManager.__init__(self, context)
        self.yum = yum.YumBase()
        self.yum.doGenericSetup(cache=1)

    def __make_package(self, pkg):
        pkg_installed = (self.yum.rpmdb.searchNames(names=[pkg.name]) or [None])[0]
        p = Package(self)
        p.id = '%s.%s' % (pkg.name, pkg.arch)
        p.name = pkg.name
        p.version = pkg.version
        p.description = pkg.arch # nothing better
        p.is_installed = pkg_installed is not None
        if p.is_installed:
            p.installed_version = pkg_installed.version
            p.is_upgradeable = p.installed_version != p.version
        return p

    def list(self):
        for pkg in self.yum.pkgSack.returnPackages():
            yield self.__make_package(pkg)

    def get(self, id):
        pkg = (self.yum.searchNames(names=[id]) or [None])[0]
        return self.__make_package(pkg)

    def update_lists(self, progress_callback):
        class Progress (object):
            def end(self, amount_read, now=None):
                pass

            def start(self, filename=None, url=None, basename=None, size=None, now=None, text=None):
                self.size = size
                self.done = 0
                self.name = url

            def update(self, amount_read, now=None):
                self.done = amount_read
                message = '%s%% %s' % (int(100 * self.done / self.size), self.name)
                progress_callback(message=message, done=self.done, total=self.size)

        progress_callback(message='Preparing')
        y = yum.YumBase()
        y.repos.setProgressBar(Progress())
        y.cleanMetadata()
        y.repos.doSetup()
        y.repos.populateSack()

    def get_apply_cmd(self, selection):
        to_install = [sel['package']['id'] for sel in selection if sel['operation'] in ['install', 'upgrade']]
        to_remove = [sel['package']['id'] for sel in selection if sel['operation'] == 'remove']
        cmd = ''
        if len(to_install) > 0:
            cmd += 'yum install ' + ' '.join(to_install)
            if len(to_remove) > 0:
                cmd += ' && '
        if len(to_remove) > 0:
            cmd += 'yum remove ' + ' '.join(to_remove)
        return cmd



import gevent
import apt
from apt.progress.base import AcquireProgress

from aj.api import *
from aj.plugins.packages.api import PackageManager, Package


@component(PackageManager)
class APTPackageManager (PackageManager):
    id = 'apt'
    name = 'APT'

    def __init__(self, context):
        PackageManager.__init__(self, context)
        self.cache = apt.Cache()

    def __make_package(self, apt_package):
        p = Package(self)
        p.id = apt_package.fullname if hasattr(apt_package, 'fullname') else apt_package.name
        p.name = p.id
        v = apt_package.versions[-1]
        p.version = v.version
        p.description = v.summary
        p.is_installed = apt_package.installed is not None
        if p.is_installed:
            p.installed_version = apt_package.installed.version
            p.is_upgradeable = p.installed_version != p.version
        return p

    def list(self):
        for id in self.cache.keys():
            yield self.__make_package(self.cache[id])

    def get(self, id):
        return self.__make_package(self.cache[id])

    def update_lists(self, progress_callback):
        class Progress (AcquireProgress):
            def fetch(self, item):
                message = '%s%% %s' % (int(100 * self.current_items / self.total_items), item.shortdesc)
                progress_callback(message=message, done=self.current_items, total=self.total_items)

            def stop(self):
                self.done = True

        ack = Progress()
        self.cache.update(fetch_progress=ack)

        while not hasattr(ack, 'done'):
            gevent.sleep(1)

    def get_apply_cmd(self, selection):
        cmd = 'apt-get install '
        for sel in selection:
            cmd += sel['package']['id'] + {'remove': '-', 'install': '+', 'upgrade': '+'}[sel['operation']] + ' '
        return cmd



import json

import aj
from aj.api import *
from aj.api.http import url, HttpPlugin

from aj.plugins.core.api.endpoint import endpoint
from aj.plugins.packages.api import PackageManager


@component(HttpPlugin)
class Handler (HttpPlugin):
    def __init__(self, context):
        self.context = context
        self.managers = dict((x.id, x) for x in PackageManager.all(self.context))

    def __package_to_json(self, package):
        return {
            'id': package.id,
            'name': package.name,
            'version': package.version,
            'description': package.description,
            'managerId': package.manager.id,
            'isInstalled': package.is_installed,
            'isUpgradeable': package.is_upgradeable,
            'installedVersion': package.installed_version,
        }

    @url(r'/api/packages/managers')
    @endpoint(api=True)
    def handle_api_managers(self, http_context):
        return [
            {
                'id': mgr.id,
                'name': mgr.name,
            } for mgr in self.managers.values()
        ]

    @url(r'/api/packages/list/(?P<manager_id>\w+)')
    @endpoint(api=True)
    def handle_api_list(self, http_context, manager_id=None):
        query = http_context.query['query']
        if len(query) < 3:
            raise Exception('Query too short')
        query = query.lower().strip()
        return [
            self.__package_to_json(package)
            for package in self.managers[manager_id].list()
            if query in package.id or query in package.name
        ]

    @url(r'/api/packages/get/(?P<manager_id>\w+)/(?P<package_id>\w+)')
    @endpoint(api=True)
    def handle_api_get(self, http_context, manager_id=None, package_id=None):
        return self.__package_to_json(self.managers[manager_id].get(package_id))

    @url(r'/api/packages/apply/(?P<manager_id>\w+)')
    @endpoint(api=True)
    def handle_api_apply(self, http_context, manager_id=None):
        mgr = self.managers[manager_id]
        selection = json.loads(http_context.body)
        cmd = mgr.get_apply_cmd(selection)
        return {
            'terminalCommand': cmd,
        }
